const {
	      byteArray,
	      gi  : { Gio: { Subprocess, SubprocessFlags }, GLib },
	      ui  : {
		      main     : {
			      sessionMode,
			      panel: { statusArea: { aggregateMenu: { _powerProfiles: { _item: { menu } } } } }
		      },
		      popupMenu: { PopupSeparatorMenuItem, PopupSwitchMenuItem }
	      },
	      misc: { extensionUtils }
      } = imports;

let mySeparator;
let mySwitch;
let myMenuConnection;
let mySwitchConnection;
let myTimeout;
// 0: intel_pstate
// 1: acpi-cpufreq
// 2: CPU not supported
let myCpuType = 2;
let initial   = true;

function init()
{
	extensionUtils.initTranslations();
}

function enable()
{
	// GNOME calls enable on every unlock. This code here needs to run only the first time, on user log-in
	if ( initial )
	{
		const state    = getBoostState();
		const settings = extensionUtils.getSettings();

		// Make sure we do not come here this session
		initial = false;

		// Turn off if it is on now, user has opted-in for persistence and the last time he turned off Boost
		if ( state && settings.get_boolean( 'persist' ) && !settings.get_boolean( 'boost' ) )
		{
			myTimeout = GLib.timeout_add( GLib.PRIORITY_DEFAULT, 5000, _ => setBoostState( false ) );
		}
		else
		{
			// Always set state. This clears use cases of this sort:
			// 1. Disable Boost
			// 2. Reboot (this will enable Boost)
			// 3. Enable Persist (while Boost is on)
			// 4. Reboot
			// User will be prompted to disable Boost since setting is remembered from the last time he clicked the
			// switch. Clear it here, so that doesn't happen.
			settings.set_boolean( 'boost', state );
		}
	}

	myMenuConnection = menu.connect( 'open-state-changed', ( { box }, open ) =>
	{
		if ( open )
		{
			const state = getBoostState();

			// Do not add switch if CPU not supported
			if ( state !== null )
			{
				mySeparator = new PopupSeparatorMenuItem;
				mySwitch    = new PopupSwitchMenuItem( extensionUtils.gettext( 'Frequency Boost' ), state );

				mySwitchConnection = mySwitch.connect( 'toggled', ( { _switch: { state } } ) => setBoostState( state ) );

				box.insert_child_at_index( mySeparator, 0 );
				box.insert_child_at_index( mySwitch, 0 );
			}
		}
		else
		{
			destroyMyItems();
		}
	} );
}

function disable()
{
	// Redundancy. Items are destroyed when menu closes, but still...
	destroyMyItems();

	menu.disconnect( myMenuConnection );
	GLib.Source.remove( myTimeout );

	myMenuConnection = myTimeout = null;

	// If user disabled the extension, bring back Boost to on (and save boolean)
	if ( sessionMode.currentMode === 'user' && !getBoostState() )
	{
		setBoostState( true );
	}
}

function getBoostState()
{
	return [
		// 0 means Boost is on, 1 means it is off
		[ 'intel_pstate/no_turbo', '0' ],
		// 0 means Boost is off, 1 means it is on
		[ 'cpufreq/boost', '1' ]
	].reduce( ( result, [ setting, enabledValue ], index ) =>
	{
		if ( result === null )
		{
			const out = GLib.spawn_command_line_sync( `bash -c "cat /sys/devices/system/cpu/${setting}"` )[ 1 ];

			if ( out.length )
			{
				myCpuType = index;

				return byteArray.toString( out ).trim() === enabledValue;
			}
		}

		return result;
	}, null );
}

function setBoostState( state )
{
	// Moved to the set_boost file. Reason - more meaningful pkexec message
	/*const [ setting, value ] = [
	 [ 'intel_pstate/no_turbo', Number( !state ) ],
	 [ 'cpufreq/boost', Number( state ) ]
	 ][ myCpuType ];

	 GLib.spawn_command_line_async( `pkexec bash -c "echo ${value} > /sys/devices/system/cpu/${setting}"` );*/

	// Wait for async pkexec, so we can save boost state only if user approved the change
	Subprocess.new(
		[
			'pkexec',
			extensionUtils.getCurrentExtension().dir.get_child( 'set_boost' ).get_path(),
			// Subprocess is not happy if we leave those as numbers
			myCpuType.toString(),
			Number( state ).toString()
		],
		SubprocessFlags.NONE
	).wait_check_async( null, ( proc, result ) =>
	{
		try
		{
			if ( proc.wait_check_finish( result ) )
			{
				// Save new state. Used at login, in conjunction with to persist flag
				extensionUtils.getSettings().set_boolean( 'boost', state );
			}
		}
		catch
		{
			// We come here when user cancels the permissions' dialog. Do not spam the log.
		}
	} );

	// Destroy init timeout
	return false;
}

function destroyMyItems()
{
	mySwitch?.disconnect( mySwitchConnection );

	mySeparator?.destroy();
	mySwitch?.destroy();

	mySeparator = mySwitch = mySwitchConnection = null;
}